from waitress import serve
from api.api import API
from whitenoise import WhiteNoise
import os
def render(app_routes,host="127.0.0.1",port=8001,url_prefix=""):
    # all other app configs to be written here
    curr = os.getcwd()
    static = os.path.join(curr,"router")
    app_ = API(app_routes)
    app_ = WhiteNoise(app_,root=static)
    app_.add_files(static,prefix="/")
    """
    app_.add_files(path,prefix='')
    """
    #print(f"======APP Routes :: {app.routes}")
    serve(app_,host=host,port=port,url_prefix=url_prefix)


from dom.dom_ import Dom
import inspect
d = Dom()
clk = str(inspect.getsource(d.generate_js_file))
clk = clk.replace(":","{")
clk = clk.replace("def","function")
clk = clk + "}"
with open("test.txt","w") as f:
    f.write(clk)
print(clk)


# models functions

class Models:
    def __init__(self,model_name,db="",dialect=""):
        self.model_name =model_name
        self.cursor = db
    # validate if the user key_queries are accurate
    """
    check if the key value pairs of queries match eg if a field is null but
    its required thru model declaration then raise an error
    """
    def _validate_input(self):
        pass
    # args is a dict of model fields
    def create(self,args):
        pass
    # used to unpack the args it returns the col names
    def _unpack_args(self,args):
        pass
    def findOne(self,arg):
        pass
    
    def findAll(self,arg):
        pass
    # to delete
    def destroy(self,arg):
        pass
    
    def findAndUpdate(self,args):
        pass
    # used to create fields for the db, not used to create tables created thru migrations
    def init(self,model_name,args):
        query = "create table query"
        # table_fields = ""
        # len_args = len(args)
        # sql = ""
    
    def _gettype_(self,arg):
        if len(arg)==0:
            return False
        if isinstance(arg,str):
            return "%s"
        elif isinstance(arg,int):
            return "%d"
    """
    hasOne and has Many are used to define associations
    model arg --> is a string name for a Model
    """
    def hasOne(self,model,options={}):
        pass
    
    def hasMany(self,model,options={}):
        pass

class DBConnection:
    """
    dialect defines the type of sql database
    args define a dictionary of db configurations eg host,port,password
    try to connect to db else raise an error
    -->install the driver for the selected dialect except for sqlite which is built in
    """
    
    def __init__(self,dialect,**kwargs):
        self.dialect = dialect
        self.kwargs = kwargs
    
    def connect(self):
        # call appropriate method based on provided dialect
        if self.dialect.lower() =="sqlite" or self.dialect.lower() =="sqlite3":
            self.sqlite_db()
        elif self.dialect.lower() =="mysql":
            pass
        pass
    def sqlite_db(self):
        import sqlite3
        # get db name from the kwargs

        conn = sqlite3.connect(self.kwargs)
        return conn
    
    def mysql_db(self):
        pass
    def postgress_db(self):
        pass
    def mssql_db(self):
        pass
    
    # other dbs


class Models_Migrate:
    pass


class ForLoop:
    
    def __init__(self,loop_struct,**kwargs):
        self.kwargs = kwargs
        self.component ="component"
        self.has_closing = True
        self.current_component = "{% if "+f"{loop_struct}"+" %}"
        self.current_end ='\n' + "{% endif %}"
        #self.current_component = current_component + self.current_end
    def __call__(self,component):
        print("============inside call")
        self.build_component()
    # append child element to the current component
    def add_child(self,child):
        print("===inside button compnent======")
        # slice from the closing tag to append the child before closing tag
        end_len = len(self.current_end)
        self.current_end =  '\n' + child + self.current_end
        print("===exit button compnent======")
    # now building the component and its attributes
    def build_component(self):
        # content of the html component
        self.current_component = self.current_component + self.current_end
        return self.current_component

def Button(arg):
    #print("===inside button compnent======")
    return f"<div>INSIDE A DIV ELEMENT {arg}</div>"
form = ForLoop("student in students")
form.add_child(Button("{{student.name}}"))
form.add_child("<div>INSIDE A DIV ELEMENT last one</div> \n")
print(form.build_component())
print()


from webob import Response,Request
import time
#from routes import Router
# from werkzeug.wrappers import Request,Response
def eg(req,*args,**kwargs):
    if req.method.lower() == "post":
            print(f"REQUEST OBJECT IS ============ {req.body}, POST == {req.POST}")
            for k in req.POST:
                print(req.POST[k])
    form = """
    <form method='post' action='/man'>
        <input type='text' name='first_name' />
        <input type='submit' value='Test Submit' />
    </form>
    """
    return form
class API:
    def __init__(self,app_routes):
        self.routes = app_routes.routes
    
    def __call__(self,environ,start_response):
        req = Request(environ)
        res = self.handle_request(req)
        return res(environ,start_response)
    
    
    def handle_request(self,req):
        
        res = Response()
        handler,handler_ = self.find_handler(req_path=req.path)
        if handler_:
            value=handler(req)
            print("====ec=xecutng value")
            res.text = value
        elif handler_==False:
            res.text = "File not foundr"
        return res
    def find_handler(self,req_path):
        while len(self.routes.keys()) >0:
            for path,handler in self.routes.items():
                if path == req_path:
                    return [handler,True]
            break
        return ["",False]




from server import render
from shortcuts.load_file import current_dir
from router.route_ import Router
print(current_dir)
import os
app = Router()
def hel(req):
    img = os.path.join(os.getcwd(),"router","section.png")
    print("=====App Mounted=======")
    print(os.getcwd())
    return "<h1>Hello There</h1>" + "<br>" + f"<img src='/router/section.png' alt='hello' />"
app.route("/",hel)
render(app)